use glam::{Mat4, Quat, Vec3};

pub struct Transform {
    position: Vec3,
    rotation: Quat,
    scale: Vec3,
}

impl Default for Transform {
    fn default() -> Self {
        Self {
            position: Vec3::default(),
            rotation: Quat::default(),
            scale: Vec3::ONE,
        }
    }
}

impl Transform {
    pub fn rotate(&mut self, rotation: Quat) {
        self.rotation = self.rotation * rotation;
    }

    pub fn translate(&mut self, translation: Vec3) {
        self.position += translation;
    }

    pub fn scale(&mut self, scale: Vec3) {
        self.scale *= scale;
    }

    pub fn get_mat4(&self) -> Mat4 {
        Mat4::from_translation(self.position)
            * Mat4::from_quat(self.rotation)
            * Mat4::from_scale(self.scale)
    }
}
