use super::{Scene, SceneContext};

pub struct SceneManager {
    current_scene: Option<Box<dyn Scene>>,
}

impl SceneManager {
    pub fn new() -> Self {
        Self {
            current_scene: None,
        }
    }

    pub fn load_scene_if_not_loaded(&mut self, scene_context: &SceneContext) {
        if let Some(current_scene) = self.current_scene.as_mut() {
            if !current_scene.loaded() {
                if let Err(e) = current_scene.load(scene_context) {
                    log::error!("Error loading scene: {}", e);
                }
            }
        }
    }

    pub fn load_scene(&mut self, scene: Box<dyn Scene>) {
        if let Some(current_scene) = self.current_scene.as_mut() {
            current_scene.unload();
        }
        self.current_scene = Some(scene);
    }

    pub fn current_scene(&self) -> Option<&Box<dyn Scene>> {
        if let Some(current_scene) = self.current_scene.as_ref() {
            if current_scene.loaded() {
                return Some(current_scene);
            }
        }
        None
    }

    pub fn current_scene_mut(&mut self) -> Option<&mut Box<dyn Scene>> {
        if let Some(current_scene) = self.current_scene.as_mut() {
            if current_scene.loaded() {
                return Some(current_scene);
            }
        }
        None
    }
}
