use std::{error::Error, sync::Arc};

use egui_winit_vulkano::Gui;
use vulkano::{image::view::ImageView, sync::GpuFuture};

mod context;
pub use context::SceneContext;

mod manager;
pub use manager::SceneManager;

pub trait Scene {
    fn loaded(&self) -> bool;
    fn load(&mut self, scene_context: &SceneContext) -> Result<(), Box<dyn Error>>;
    fn update(&mut self, scene_context: &SceneContext) -> Result<(), Box<dyn Error>>;
    fn render(
        &mut self,
        image_view: &Arc<ImageView>,
        acquire_future: Box<dyn GpuFuture>,
        scene_context: &SceneContext,
        gui: &mut Gui,
    ) -> Result<Box<dyn GpuFuture>, Box<dyn Error>>;
    fn unload(&mut self);
}
